﻿
Imports System.CodeDom.Compiler
Imports System.Reflection
Imports System.Collections.Generic
Imports System.Collections.ObjectModel


' ===============================================================================================================
'   Access to HOST objects
' ===============================================================================================================

Public Module Host

    Public ReadOnly Property GetPictureBox() As PictureBox
        Get
            Return Form_Main.PictureBox1
        End Get
    End Property

    Public ReadOnly Property GetTextBox() As TextBox
        Get
            Return Form_Main.TextBox1
        End Get
    End Property

    Public Sub ShowMessage(ByVal Message As String)
        MessageBox.Show(Message)
    End Sub

End Module




' ===============================================================================================================
'  COMPILER FUNCTIONS
' ===============================================================================================================

Module Module_Compiler

    Friend Enum Compiler_Languages
        VB
        CSharp
    End Enum

    Friend Compiler_Language As Compiler_Languages = Compiler_Languages.VB
    Friend Compiler_WarningLevel As Int32 = -1
    Friend Compiler_HideHeader As Boolean = True
    Friend Compiler_AddDebugInfo As Boolean = False
    Friend Compiler_UseConsole As Boolean = False
    Friend Compiler_Options As String = ""
    Friend Compiler_HeaderLinesCount As Int32

    Friend Compiler_ErrorLines As HashSet(Of Int32) = New HashSet(Of Int32)

    Friend Compiler_Results As CompilerResults
    Friend Compiler_ErrorCount As Int32
    Friend Compiler_WarningCount As Int32


    'http://msdn.microsoft.com/it-it/library/c03639ab(VS.80).aspx

    Friend Sub Compiler_SetOptions(ByVal _Language As Compiler_Languages, _
                                   ByVal _WarningLevel As Int32, _
                                   ByVal _HideHeader As Boolean, _
                                   ByVal OptionExplicit As Boolean, _
                                   ByVal OptionStrict As Boolean, _
                                   ByVal OptionOptimize As Boolean, _
                                   ByVal _AddDebugInfo As Boolean, _
                                   ByVal _UseConsole As Boolean)
        Compiler_Language = _Language
        Compiler_WarningLevel = _WarningLevel
        Compiler_HideHeader = _HideHeader
        Compiler_AddDebugInfo = _AddDebugInfo
        Compiler_UseConsole = _UseConsole
        Compiler_Options = ""

        If _Language = Compiler_Languages.VB Then
            Editor_CommentChars = "'"
            If OptionExplicit Then Compiler_Options &= " /OptionExplicit" Else Compiler_Options &= " /OptionExplicit-"
            If OptionStrict Then Compiler_Options &= " /OptionStrict" Else Compiler_Options &= " /OptionStrict-"
        Else
            Editor_CommentChars = "//"
        End If

        'If OptionNoWarn Then Compiler_Options &= " /NoWarn"
        'If OptionInfer Then Compiler_Options &= " /optioninfer+" Else Compiler_Options &= " /optioninfer-"
        If OptionOptimize Then Compiler_Options &= " /Optimize" Else Compiler_Options &= " /Optimize-"

    End Sub



    ' todo explore "CodeDomProvider.Parse"

    ' -------------- Good compile example ------------------------------------------------
    ' http://msdn.microsoft.com/it-it/library/system.codedom.compiler.codedomprovider.aspx

    '   /t[arget]:lib[rary]   Create a library
    '   /t[arget]:exe         Create a console executable
    '   /t[arget]:win[exe]    Create a Windows executable

    Friend Sub Compiler_CompileText(ByVal ScriptText As String, _
                                    ByVal FileName As String, _
                                    ByVal CompileInMemory As Boolean, _
                                    ByVal CompileLibrary As Boolean, _
                                    Optional ByVal ExcludeErrorLines As Boolean = False)

        ' ----------------------------------------------------------------------- select provider
        Dim provider As CodeDomProvider = Nothing
        Select Case Compiler_Language
            Case Compiler_Languages.VB : provider = New Microsoft.VisualBasic.VBCodeProvider()
            Case Compiler_Languages.CSharp : provider = New Microsoft.CSharp.CSharpCodeProvider()
        End Select

        ' ----------------------------------------------------------------------- configure parameters
        ' --------------------------------------------------------------------------------------------------
        ' Per specificare una stringa di argomenti personalizzata della riga di comando da utilizzare quando
        '   si richiama il processo di compilazione, impostare la stringa nella proprietà CompilerOptions. 
        ' Se è necessario un token di sicurezza Win32 per richiamare il processo di compilazione, 
        '   specificare il token nella proprietà UserToken. 
        ' Per includere file di risorse .NET Framework nell'assembly compilato, 
        '   aggiungere i nomi dei file di risorse alla proprietà EmbeddedResources.
        ' Per fare riferimento a risorse .NET Framework in un altro assembly, 
        '   aggiungere i nomi dei file di risorse alla proprietà LinkedResources.
        ' Per includere un file di risorse Win32 nell'assembly compilato,
        '   specificare il nome del file di risorse Win32 nella proprietà Win32Resource.
        ' --------------------------------------------------------------------------------------------------
        Dim params As CompilerParameters = New CompilerParameters
        With params
            .WarningLevel = Compiler_WarningLevel   ' -1=default / 0=less / 5=more
            .MainClass = "Script"

            If CompileLibrary Then
                .CompilerOptions = Compiler_Options & " /target:library"
                .GenerateExecutable = False                         ' < create DLL
            Else
                If Compiler_UseConsole Then
                    .CompilerOptions = Compiler_Options & " /target:exe"
                Else
                    .CompilerOptions = Compiler_Options & " /target:winexe"
                End If
                .GenerateExecutable = True                          ' < create EXE
            End If

            If CompileInMemory Then
                .OutputAssembly = ""                                ' use "Complete path" & "Script.dll" to write a dll
                .GenerateInMemory = True                            ' false to create DLL file
                .IncludeDebugInformation = False                    ' true to create PDB file
            Else
                .OutputAssembly = FileName                          ' use "Complete path" & "Script.dll" to write a dll
                .GenerateInMemory = False                           ' false to create DLL file
                .IncludeDebugInformation = Compiler_AddDebugInfo    ' true to create PDB file
                '
                '.CompilerOptions = .CompilerOptions & " /title:" & """PIPPO"""
            End If


            ' ---------------------------------------------------- add all the assemblies of the current application
            '
            For Each ASM As Assembly In AppDomain.CurrentDomain.GetAssemblies()
                If ASM.Location <> "" Then
                    params.ReferencedAssemblies.Add(ASM.Location)
                End If
                'Debug.Print(ASM.FullName)
            Next

        End With


        ' ----------------------------------------------------------------------- special compilation without errors
        If ExcludeErrorLines Then

            Dim lines() As String = Split(ScriptText, vbLf)

            Dim TextWithoutErrorLines As String = ""
            For line As Int32 = 0 To lines.Count - 1
                If Not Compiler_ErrorLines.Contains(line) Then
                    TextWithoutErrorLines &= lines(line) & vbCrLf
                End If
            Next

            Compiler_Results = provider.CompileAssemblyFromSource(params, _
                                                                  AddHeaderAndFooter(TextWithoutErrorLines))
            provider.Dispose()
            Exit Sub
        End If
        ' ----------------------------------------------------------------------- exit here without error test




        ' ----------------------------------------------------------------------- normal compilation
        Compiler_Results = provider.CompileAssemblyFromSource(params, _
                                                              AddHeaderAndFooter(ScriptText))

        provider.Dispose()


        ' ----------------------------------------------------------------------- test for errors
        Compiler_ErrorLines.Clear()
        Compiler_ErrorCount = 0
        Compiler_WarningCount = 0
        For Each err As CompilerError In Compiler_Results.Errors
            If err.IsWarning Then
                Compiler_WarningCount += 1
            Else
                Compiler_ErrorCount += 1
                Compiler_ErrorLines.Add(err.Line - Compiler_HeaderLinesCount - 1)
            End If
        Next



        If CompileInMemory Then
            ' -------------------------------------------------------------------------------------
            '  TODO use a separate AppDomain
            ' -------------------------------------------------------------------------------------
            'Dim newDomain As AppDomain = AppDomain.CreateDomain("newDomain")
            'newDomain.Load("Script")
            'AppDomain.Unload(newDomain)

            ' -------------------------------------------------------------------------------------
            '  using "Delegate"
            ' -------------------------------------------------------------------------------------
            If Compiler_ErrorCount = 0 Then
                Main = Nothing
                Dim DynamicClass As Type = Compiler_Results.CompiledAssembly.GetType("Script")
                If DynamicClass IsNot Nothing Then

                    'Dim b As BindingFlags = Reflection.BindingFlags.Public Or _
                    '                        Reflection.BindingFlags.Static
                    Dim b As BindingFlags = BindingFlags.Public Or _
                                            BindingFlags.NonPublic Or _
                                            BindingFlags.Static Or _
                                            BindingFlags.Instance Or _
                                            BindingFlags.IgnoreCase

                    Dim DynamicFunction As Reflection.MethodInfo = DynamicClass.GetMethod("Main", b)
                    If DynamicFunction IsNot Nothing Then
                        Try
                            Main = CType([Delegate].CreateDelegate(GetType(Main_Delegate), Nothing, DynamicFunction), Main_Delegate)
                        Catch ex As Exception
                            Editor_ShowLinkerError(" ""Script Main""  " & ex.Message)
                        End Try
                    Else
                        Editor_ShowLinkerError(" ""Script Main"" not found.")
                    End If
                Else
                    Editor_ShowLinkerError(" Class ""Script"" not found.")
                End If
            End If
        End If

    End Sub



    ' -------------------------------------------------------------------------------------
    '  using "Delegate"
    ' -------------------------------------------------------------------------------------

    ' --------------------------------------------- this delegate must match the signature in Script-Code
    Private Delegate Sub Main_Delegate(ByVal args() As String)
    Dim Main As Main_Delegate

    Friend Sub Compiler_RunInMemory(ByVal CsvArgsString As String)
        '
        If Main Is Nothing Then
            Beep()
            Exit Sub
        End If
        ' ----------------------------------------------------- Save before Run
        Editor_Save()
        ' ----------------------------------------------------- Run
        Try
            Dim args(-1) As String
            If CsvArgsString IsNot Nothing Then args = Split(CsvArgsString, ",")
            Main(args)
        Catch ex As Exception
            Editor_ShowRuntimeError(ex.ToString)
        End Try
    End Sub

    Friend Sub Compiler_RunExeFile(ByVal ExeFileName As String)
        ' ----------------------------------------------------- Save before Run
        Editor_Save()
        ' ----------------------------------------------------- Run
        Dim p As Process = Nothing
        Try
            Editor_RunningState_Set()
            p = Process.Start(ExeFileName)
            Do
                Application.DoEvents()
                System.Threading.Thread.Sleep(100)
            Loop Until p.HasExited
        Catch ex As Exception
            If p IsNot Nothing Then
                p.Close()
            End If
            Editor_ShowRuntimeError(ex.ToString)
        End Try
        Editor_RunningState_Reset()
    End Sub

    'Friend Sub Compiler_RunExeFile(ByVal ExeFileName As String)
    '    ' ----------------------------------------------------- Save before Run
    '    Editor_Save()
    '    ' ----------------------------------------------------- Run
    '    Try
    '        Editor_RunningState_Set()
    '        Dim otherDomain As AppDomain = AppDomain.CreateDomain("ThereminoScriptDomain")
    '        otherDomain.ExecuteAssembly(ExeFileName)
    '        AppDomain.Unload(otherDomain)
    '        Editor_RunningState_Reset()
    '    Catch ex As Exception
    '        Editor_ShowRuntimeError(ex.ToString)
    '    End Try
    'End Sub



    Private Function AddHeaderAndFooter(ByVal ScriptText As String) As String

        If Not Compiler_HideHeader Then
            Compiler_HeaderLinesCount = 0
            Return ScriptText & Compiler_AddMemoryMappedFiles(Compiler_Language)
        End If


        Dim header As String = Compiler_GetImports(Compiler_Language)
        Dim footer As String = ""

        If Compiler_Language = Compiler_Languages.VB Then
            ' ----------------------------------------------------------------------- script header 
            header &= "Module Script" & vbLf
            ' ----------------------------------------------------------------------- script footer
            footer = "End Module" & vbLf
        Else
            ' ----------------------------------------------------------------------- script header 
            header &= "partial class Script" & vbLf
            header &= "{" & vbLf
            ' ----------------------------------------------------------------------- script footer
            footer = "}" & vbLf
        End If
        '
        Compiler_HeaderLinesCount = Split(header, vbLf).Count - 1
        '
        Return header & ScriptText & vbLf & footer & Compiler_AddMemoryMappedFiles(Compiler_Language)
    End Function

    Friend Function Compiler_GetEmptyNewText(ByVal language As Compiler_Languages) As String
        Compiler_GetEmptyNewText = vbLf & Compiler_GetImports(language) & vbLf
        Compiler_GetEmptyNewText &= Compiler_GetAssemblyOptions(language) & vbLf
        If language = Compiler_Languages.VB Then
            Compiler_GetEmptyNewText &= _
                 "Public Class Script" & vbLf & _
                 "    Public Shared Sub Main(ByVal s() As String)" & vbLf & _
                 "        MessageBox.Show(""Theremino Script"")" & vbLf & _
                 "    End Sub" & vbLf & _
                 "End Class"
        Else
            Compiler_GetEmptyNewText &= _
                "public class Script" & vbLf & _
                "{" & vbLf & _
                "    public static void Main(String[] s) " & vbLf & _
                "    {" & vbLf & _
                "        MessageBox.Show(""Theremino Script"");" & vbLf & _
                "    }" & vbLf & _
                "}"
        End If
    End Function


    Private Function Compiler_GetImports(ByVal language As Compiler_Languages) As String
        If language = Compiler_Languages.VB Then
            Return "Imports Microsoft.VisualBasic" & vbLf & _
                    "Imports System" & vbLf & _
                    "Imports System.Windows.Forms" & vbLf & _
                    "Imports System.Drawing" & vbLf & _
                    "Imports System.Math" & vbLf & _
                    "Imports System.ComponentModel" & vbLf & _
                    "Imports System.Diagnostics" & vbLf & _
                    "Imports System.Web" & vbLf & _
                    "Imports System.Configuration" & vbLf & _
                    "Imports System.Net" & vbLf & _
                    "Imports System.Net.Sockets" & vbLf & _
                    "Imports System.Text" & vbLf & _
                    "Imports System.Text.RegularExpressions" & vbLf & _
                    "Imports System.Collections" & vbLf & _
                    "Imports System.Collections.Generic" & vbLf & _
                    "Imports System.Reflection" & vbLf & _
                    "Imports System.Runtime.InteropServices" & vbLf & _
                    "Imports Theremino_Script" & vbLf
        Else
            Return "using Microsoft.VisualBasic; // ( to use VB functions like ""InputBox"" )" & vbLf & _
                    "using System;" & vbLf & _
                    "using System.Windows.Forms;" & vbLf & _
                    "using System.Drawing;" & vbLf & _
                    "using System.ComponentModel;" & vbLf & _
                    "using System.Diagnostics;" & vbLf & _
                    "using System.Web;" & vbLf & _
                    "using System.Configuration;" & vbLf & _
                    "using System.Net;" & vbLf & _
                    "using System.Net.Sockets;" & vbLf & _
                    "using System.Text.RegularExpressions;" & vbLf & _
                    "using System.Text;" & vbLf & _
                    "using System.Collections;" & vbLf & _
                    "using System.Collections.Generic;" & vbLf & _
                    "using System.Reflection;" & vbLf & _
                    "using System.Runtime.InteropServices;" & vbLf & _
                    "using Theremino_Script;" & vbLf
        End If
    End Function

    Private Function Compiler_GetAssemblyOptions(ByVal language As Compiler_Languages) As String
        If language = Compiler_Languages.VB Then
            Return "<Assembly: AssemblyTitle(""ScriptingTest"")>" & vbLf & _
                    "<Assembly: AssemblyDescription("""")>" & vbLf & _
                    "<Assembly: AssemblyCompany("""")>" & vbLf & _
                    "<Assembly: AssemblyProduct(""Theremino Script"")>" & vbLf & _
                    "<Assembly: AssemblyCopyright(""Copyright © xxx xxx"")>" & vbLf & _
                    "<Assembly: AssemblyTrademark("""")>" & vbLf & _
                    "<Assembly: ComVisible(False)>" & vbLf & _
                    "<Assembly: Guid(""ff17d595-7720-42c5-99a4-e1fe5dd91f92"")>" & vbLf & _
                    "<Assembly: AssemblyVersion(""1.0.0.0"")>" & vbLf & _
                    "<Assembly: AssemblyFileVersion(""1.0.0.0"")>" & vbLf
        Else
            Return "[assembly: AssemblyTitle(""title"")]" & vbLf & _
                    "[assembly: AssemblyDescription("""")]" & vbLf & _
                    "[assembly: AssemblyCompany("""")]" & vbLf & _
                    "[assembly: AssemblyProduct(""Theremino Script"")]" & vbLf & _
                    "[assembly: AssemblyCopyright(""Copyright © xxx xxx"")]" & vbLf & _
                    "[assembly: AssemblyTrademark("""")]" & vbLf & _
                    "[assembly: ComVisible(false)]" & vbLf & _
                    "[assembly: Guid(""ff17d595-7720-42c5-99a4-e1fe5dd91f92"")]" & vbLf & _
                    "[assembly: AssemblyVersion(""1.0.0.0"")]" & vbLf & _
                    "[assembly: AssemblyFileVersion(""1.0.0.0"")]" & vbLf
        End If
    End Function


    Private Function Compiler_AddMemoryMappedFiles(ByVal language As Compiler_Languages) As String
        If language = Compiler_Languages.VB Then
            Return My.Resources.Module_MemoryMappedFiles_VB
        Else
            Return My.Resources.Module_MemoryMappedFiles_CS
        End If
    End Function

End Module


